package net.knarcraft.paidsigns.listener;

import net.knarcraft.paidsigns.PaidSigns;
import net.knarcraft.paidsigns.container.PaidSign;
import net.knarcraft.paidsigns.formatting.StringFormatter;
import net.knarcraft.paidsigns.formatting.TranslatableMessage;
import net.knarcraft.paidsigns.manager.EconomyManager;
import net.knarcraft.paidsigns.manager.TrackedSignManager;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.block.SignChangeEvent;

import java.io.IOException;
import java.util.Map;

/**
 * A listener for listening to registered paid signs
 */
public class SignListener implements Listener {

    @EventHandler(priority = EventPriority.LOW)
    @SuppressWarnings("unused")
    public void onSignChange(SignChangeEvent event) {
        if (event.isCancelled() || event.getPlayer().hasPermission("paidsigns.paymentexempt")) {
            return;
        }

        String[] lines = event.getLines();
        Map<String, PaidSign> matchingSigns = PaidSigns.getInstance().getSignManager().getAllPaidSigns();
        for (PaidSign paidSign : matchingSigns.values()) {
            //If a match is found, just return
            if (matchSign(paidSign, lines, event)) {
                return;
            }
        }
    }

    /**
     * Checks if the given paid sign matches the given sign lines
     *
     * @param paidSign <p>The paid sign to test against the sign lines</p>
     * @param lines    <p>The lines of a sign</p>
     * @param event    <p>The triggered sign change event to cancel if necessary</p>
     * @return <p>True if a match was found and actions have been taken</p>
     */
    private boolean matchSign(PaidSign paidSign, String[] lines, SignChangeEvent event) {
        if (paidSign.matches(lines)) {
            Player player = event.getPlayer();
            String permission = paidSign.getPermission();
            //If a match is found, but the player is missing the permission, assume no plugin sign was created
            if (permission != null && !permission.trim().isEmpty() && !player.hasPermission(permission)) {
                return true;
            }

            performPaidSignTransaction(paidSign, player, event);
            return true;
        }
        return false;
    }

    /**
     * Performs the transaction to pay for the paid sign
     *
     * @param paidSign <p>The paid sign a match has been found for</p>
     * @param player   <p>The player that created the sign</p>
     * @param event    <p>The sign change event that caused the sign to be created</p>
     */
    private void performPaidSignTransaction(PaidSign paidSign, Player player, SignChangeEvent event) {
        double cost = paidSign.getCost();
        boolean canAfford = EconomyManager.canAfford(player, cost);
        if (!canAfford) {
            player.sendMessage(replaceCost(cost, StringFormatter.getTranslatedErrorMessage(
                    TranslatableMessage.ERROR_CANNOT_AFFORD)));
            event.setCancelled(true);
        } else {
            EconomyManager.withdraw(player, cost);
            player.sendMessage(replaceCost(cost, StringFormatter.getTranslatedInfoMessage(
                    TranslatableMessage.SUCCESS_PAID_FOR_SIGN)));
            try {
                TrackedSignManager.addTrackedSign(event.getBlock().getLocation(), player.getUniqueId(), cost);
            } catch (IOException ignored) {
            }
        }
    }

    /**
     * Replaces cost and unit placeholders in the given message
     *
     * @param cost    <p>The cost to insert</p>
     * @param message <p>The original message to replace the cost placeholder for</p>
     * @return <p>The message with the cost instead of the cost placeholder</p>
     */
    private String replaceCost(double cost, String message) {
        String unit = EconomyManager.getCurrency(cost != 1);
        return String.format(StringFormatter.replacePlaceholders(message, new String[]{"{cost}", "{unit}"},
                new String[]{"%.2f", "%s"}), cost, unit);
    }

}
